var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const path_1 = require("path");
const node_core_1 = require("@speedy/node-core");
const json_extends_1 = require("@speedy/json-extends");
const rules_1 = require("../rules");
var validator;
(function (validator) {
    const DEFAULT_CONFIG_FILENAME = "speedy-commit-msg.json";
    const INTERNAL_CONFIG_LOCATION = path_1.join(__dirname, "../../config");
    function validate(commitMessage, configFilePath) {
        return __awaiter(this, void 0, void 0, function* () {
            const configPath = node_core_1.config.getConfigFilePath(configFilePath || DEFAULT_CONFIG_FILENAME, INTERNAL_CONFIG_LOCATION);
            const configData = json_extends_1.json.readSync(configPath, {
                "@speedy/commit-msg-hook:latest": path_1.join(INTERNAL_CONFIG_LOCATION, DEFAULT_CONFIG_FILENAME)
            });
            const { type, message, scope, subject } = configData.rules;
            commitMessage = commitMessage || (yield getCommitMessage());
            if (message) {
                // skip validation for certain commit messages
                if (rules_1.rules.skipValidation(commitMessage, message[rules_1.rules.SKIP_VALIDATION_RULE_KEY])) {
                    return;
                }
                validatePart(commitMessage, rules_1.COMMIT_MESSAGE_PART.Message, message);
            }
            if (!rules_1.rules.SCOPED_COMMIT_REGEXP.test(commitMessage)) {
                return;
            }
            const msgSplit = /(.+)\((.+)\):\s{1}(.+)/.exec(commitMessage);
            if (!msgSplit) {
                throw new Error(`Commit message 'header' must be in this format: '<type>(<scope>): <subject>'`);
            }
            const [, msgType, msgScope, msgSubject] = msgSplit;
            if (msgType) {
                validatePart(msgType, rules_1.COMMIT_MESSAGE_PART.Type, type);
            }
            if (msgScope) {
                validatePart(msgScope, rules_1.COMMIT_MESSAGE_PART.Scope, scope);
            }
            if (msgSubject) {
                validatePart(msgSubject, rules_1.COMMIT_MESSAGE_PART.Subject, subject);
            }
        });
    }
    validator.validate = validate;
    function validatePart(text, messagePart, options) {
        _.forEach(options, (value, key) => {
            if (value === false) {
                return;
            }
            const result = _.get(rules_1.rules, _.camelCase(key))
                .call(null, text, messagePart, value);
            if (_.isObject(result) && result.failed) {
                throw new Error(result.message);
            }
        });
    }
    validator.validatePart = validatePart;
    function getCommitMessage() {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield node_core_1.fileSystem.readFileAsync(process.argv[2]);
            if (!result) {
                throw new Error("No commit message provided.");
            }
            return result.split("\n", 1)[0];
        });
    }
})(validator = exports.validator || (exports.validator = {}));
